@@ -4,6 +4,7 @@ from .context import yfc_dat as yfcd
 from .context import yfc_time as yfct
 
 import pandas as pd
+import numpy as np
 
 from datetime import datetime, date, time, timedelta
 from zoneinfo import ZoneInfo
@@ -25,7 +26,119 @@ class Test_Market_Intervals_USA(unittest.TestCase):
         self.market = "us_market"
         self.exchange = "NMS"
         self.market_tz = ZoneInfo('America/New_York')
-        self.market_close_time = time(hour=16, minute=0)
+        self.market_open_time = time(9,30)
+        self.market_close_time = time(16,0)
+
+
+    def test_GetScheduleIntervals_hourly(self):
+        interval = yfcd.Interval.Hours1
+
+        for d in [14,15,16,17,18]:
+            start_d = date(2022,2,d)
+            end_d   = date(2022,2,d+1)
+
+            response = yfct.GetExchangeScheduleIntervals(self.exchange, interval, start_d, end_d, weeklyUseYahooDef=False)
+            answer_times = [(time( 9,30), time(10,30)), 
+                            (time(10,30), time(11,30)), 
+                            (time(11,30), time(12,30)), 
+                            (time(12,30), time(13,30)), 
+                            (time(13,30), time(14,30)), 
+                            (time(14,30), time(15,30)), 
+                            (time(15,30), time(16,0))]
+            left  = [datetime.combine(start_d, t1, self.market_tz) for t1,t2 in answer_times]
+            right = [datetime.combine(start_d, t2, self.market_tz) for t1,t2 in answer_times]
+            answer = pd.IntervalIndex.from_arrays(left, right, closed="left")
+            try:
+                self.assertTrue((response==answer).all())
+            except:
+                print("response:")
+                pprint(response)
+                print("answer:")
+                pprint(answer)
+                raise
+
+            start_dt = datetime.combine(start_d, time(11,30), tzinfo=self.market_tz)
+            end_dt   = datetime.combine(start_d, time(14,30), tzinfo=self.market_tz)
+            response = yfct.GetExchangeScheduleIntervals(self.exchange, interval, start_dt, end_dt, weeklyUseYahooDef=False)
+            answer_times = [(time(11,30), time(12,30)),
+                            (time(12,30), time(13,30)),
+                            (time(13,30), time(14,30))]
+            left  = [datetime.combine(start_d, t1, self.market_tz) for t1,t2 in answer_times]
+            right = [datetime.combine(start_d, t2, self.market_tz) for t1,t2 in answer_times]
+            answer = pd.IntervalIndex.from_arrays(left, right, closed="left")
+            try:
+                self.assertTrue((response==answer).all())
+            except:
+                print("response:")
+                pprint(response)
+                print("answer:")
+                pprint(answer)
+                raise
+
+            start_dt = datetime.combine(start_d, time(11,0), tzinfo=self.market_tz)
+            end_dt   = datetime.combine(start_d, time(14,0), tzinfo=self.market_tz)
+            response = yfct.GetExchangeScheduleIntervals(self.exchange, interval, start_dt, end_dt, weeklyUseYahooDef=False)
+            answer_times = [(time(11,30), time(12,30)),
+                            (time(12,30), time(13,30))]
+            left  = [datetime.combine(start_d, t1, self.market_tz) for t1,t2 in answer_times]
+            right = [datetime.combine(start_d, t2, self.market_tz) for t1,t2 in answer_times]
+            answer = pd.IntervalIndex.from_arrays(left, right, closed="left")
+            try:
+                self.assertTrue((response==answer).all())
+            except:
+                print("response:")
+                pprint(response)
+                print("answer:")
+                pprint(answer)
+                raise
+
+    def test_GetScheduleIntervals_dayWeek(self):
+        interval = yfcd.Interval.Days1
+        start_d = date(2022,2,14) # Monday
+        end_d   = date(2022,2,26) # next Saturday
+        response = yfct.GetExchangeScheduleIntervals(self.exchange, interval, start_d, end_d, weeklyUseYahooDef=False)
+        answer_days = [date(2022,2,14), date(2022,2,15), date(2022,2,16), date(2022,2,17), date(2022,2,18), 
+                                        date(2022,2,22), date(2022,2,23), date(2022,2,24), date(2022,2,25)]
+        answer_days = np.array(answer_days)
+        answer = yfcd.DateIntervalIndex.from_arrays(answer_days, answer_days+timedelta(days=1), closed="left")
+        try:
+            self.assertTrue(response==answer)
+        except:
+            print("response:")
+            pprint(response)
+            print("answer:")
+            pprint(answer)
+            raise
+
+        intervals = [yfcd.Interval.Days5, yfcd.Interval.Week]
+        start_d = date(2022,2,14) # Monday
+        end_d   = date(2022,2,26) # next Saturday
+        answer = yfcd.DateIntervalIndex.from_arrays([date(2022,2,14),date(2022,2,22)], [date(2022,2,19),date(2022,2,26)], closed="left")
+        for interval in intervals:
+            response = yfct.GetExchangeScheduleIntervals(self.exchange, interval, start_d, end_d, weeklyUseYahooDef=False)
+            try:
+                self.assertTrue(response==answer)
+            except:
+                print("response:")
+                pprint(response)
+                print("answer:")
+                pprint(answer)
+                raise
+        # Start/end dates are in middle of week:
+        start_d = date(2022,2,9) # Wednesday
+        end_d   = date(2022,2,23) # Wednesday
+        answer = yfcd.DateIntervalIndex.from_arrays([date(2022,2,14)], [date(2022,2,19)], closed="left")
+        for interval in intervals:
+            response = yfct.GetExchangeScheduleIntervals(self.exchange, interval, start_d, end_d, weeklyUseYahooDef=False)
+            try:
+                self.assertTrue(response==answer)
+            except:
+                print("response:")
+                pprint(response)
+                print("answer:")
+                pprint(answer)
+                raise
+
 
     def test_GetTimestampCurrentInterval_open(self):
         intervals = []
@@ -38,8 +151,8 @@ class Test_Market_Intervals_USA(unittest.TestCase):
         intervals.append(yfcd.Interval.Mins90)
         intervals.append(yfcd.Interval.Hours1)
         times = []
-        times.append(time(hour=9, minute=30))
-        times.append(time(hour=15, minute=30))
+        times.append(self.market_open_time)
+        times.append(time(15, 30))
         for i in range(len(intervals)):
             interval = intervals[i]
             interval_td = yfcd.intervalToTimedelta[interval]
@@ -49,10 +162,10 @@ class Test_Market_Intervals_USA(unittest.TestCase):
                     ## dt at start of interval:
                     dt = datetime.combine(day, t, self.market_tz)
                     intervalRange = yfct.GetTimestampCurrentInterval(self.exchange, dt, interval)
-                    answer = {"interval_open":datetime.combine(day, t, self.market_tz),
+                    answer = {"interval_open":datetime.combine(day, t, self.market_tz), 
                              "interval_close":datetime.combine(day, t, self.market_tz)+interval_td}
                     if answer["interval_close"].time() > self.market_close_time:
-                        answer["interval_close"] = datetime.combine(answer["interval_close"].date(), self.market_close_time, self.market_tz)
+                        answer["interval_close"] = datetime.combine(day, self.market_close_time, self.market_tz)
                     try:
                         self.assertEqual(intervalRange, answer)
                     except:
@@ -90,8 +203,8 @@ class Test_Market_Intervals_USA(unittest.TestCase):
         intervals.append(yfcd.Interval.Days5)
         intervals.append(yfcd.Interval.Week)
         times = []
-        times.append(time(hour=9, minute=30))
-        times.append(time(hour=15, minute=30))
+        times.append(self.market_open_time)
+        times.append(time(15, 30))
         for i in range(len(intervals)):
             interval = intervals[i]
             for d in [14,15,16,17,18]:
@@ -100,11 +213,9 @@ class Test_Market_Intervals_USA(unittest.TestCase):
                     dt = datetime.combine(day, t, self.market_tz)
                     intervalRange = yfct.GetTimestampCurrentInterval(self.exchange, dt, interval)
                     if interval == yfcd.Interval.Days1:
-                        answer = {"interval_open":day,
-                                "interval_close":day+timedelta(days=1)}
+                        answer = {"interval_open":day, "interval_close":day+timedelta(days=1)}
                     else:
-                        answer = {"interval_open":date(2022,2,14),
-                                "interval_close":date(2022,2,19)}
+                        answer = {"interval_open":date(2022,2,14), "interval_close":date(2022,2,19)}
                     try:
                         self.assertEqual(intervalRange, answer)
                     except:
@@ -122,8 +233,8 @@ class Test_Market_Intervals_USA(unittest.TestCase):
         intervals.append(yfcd.Interval.Week)
         day = date(2022,2,22) # Tuesday after public holiday
         times = []
-        times.append(time(hour=8))
-        times.append(time(hour=18))
+        times.append(time(8))
+        times.append(time(18))
         answer = {"interval_open":day, "interval_close":day+timedelta(days=4)}
         for i in range(len(intervals)):
             interval = intervals[i]
@@ -149,8 +260,8 @@ class Test_Market_Intervals_USA(unittest.TestCase):
         intervals.append(yfcd.Interval.Week)
         day = date(2022,2,21) # Public holiday
         times = []
-        times.append(time(hour=8))
-        times.append(time(hour=18))
+        times.append(time(8))
+        times.append(time(18))
         answer = {"interval_open":day, "interval_close":day+timedelta(days=5)}
         for i in range(len(intervals)):
             interval = intervals[i]
@@ -175,8 +286,8 @@ class Test_Market_Intervals_USA(unittest.TestCase):
 
         ## Before/after market hours
         times = []
-        times.append(time(hour=9, minute=29))
-        times.append(time(hour=16, minute=0))
+        times.append(time(9, 29))
+        times.append(time(16, 0))
         intervals = []
         intervals.append(yfcd.Interval.Mins1)
         intervals.append(yfcd.Interval.Mins2)
@@ -206,8 +317,8 @@ class Test_Market_Intervals_USA(unittest.TestCase):
 
         ## Weekend, at times that would be open if weekday
         times = []
-        times.append(time(hour=9, minute=30))
-        times.append(time(hour=15, minute=30))
+        times.append(self.market_open_time)
+        times.append(time(15, 30))
         intervals = []
         intervals.append(yfcd.Interval.Mins1)
         intervals.append(yfcd.Interval.Mins2)
@@ -238,8 +349,8 @@ class Test_Market_Intervals_USA(unittest.TestCase):
 
         ## Public holiday, at times that would be open if weekday
         times = []
-        times.append(time(hour=9, minute=30))
-        times.append(time(hour=15, minute=30))
+        times.append(self.market_open_time)
+        times.append(time(15, 30))
         intervals = []
         intervals.append(yfcd.Interval.Mins1)
         intervals.append(yfcd.Interval.Mins2)
@@ -271,10 +382,11 @@ class Test_Market_Intervals_USA(unittest.TestCase):
         intervals = []
         intervals.append(yfcd.Interval.Days5)
         intervals.append(yfcd.Interval.Week)
+        t = time(10,0)
         ## 1) weeklyUseYahooDef=False
         for d in [19,20,21]:
             day = date(2022,2,d)
-            dt = datetime.combine(day, time(hour=10, minute=0), self.market_tz)
+            dt = datetime.combine(day, t, self.market_tz)
             for i in range(len(intervals)):
                 interval = intervals[i]
                 intervalRange = yfct.GetTimestampCurrentInterval(self.exchange, dt, interval, weeklyUseYahooDef=False)
@@ -291,7 +403,7 @@ class Test_Market_Intervals_USA(unittest.TestCase):
         ## 2) weeklyUseYahooDef=True, so self.monday_pubHol treated as in-interval
         for d in [19,20]:
             day = date(2022,2,d)
-            dt = datetime.combine(day, time(hour=10, minute=0), self.market_tz)
+            dt = datetime.combine(day, t, self.market_tz)
             for i in range(len(intervals)):
                 interval = intervals[i]
                 intervalRange = yfct.GetTimestampCurrentInterval(self.exchange, dt, interval, weeklyUseYahooDef=True)
@@ -459,8 +571,8 @@ class Test_Market_Intervals_USA(unittest.TestCase):
         intervals.append(yfcd.Interval.Mins60)
         intervals.append(yfcd.Interval.Hours1)
         times = []
-        times.append(time(hour=9, minute=30))
-        times.append(time(hour=13, minute=30))
+        times.append(self.market_open_time)
+        times.append(time(13, 30))
         for i in range(len(intervals)):
             interval = intervals[i]
             interval_td = yfcd.intervalToTimedelta[interval]
@@ -511,10 +623,10 @@ class Test_Market_Intervals_USA(unittest.TestCase):
         interval = yfcd.Interval.Mins90
         times = []
         answers = []
-        times.append(time(hour=9, minute=30))  ; answers.append({"interval_open":time(hour=11, minute=0), "interval_close":time(hour=12, minute=30)})
-        times.append(time(hour=9, minute=45))  ; answers.append({"interval_open":time(hour=11, minute=0), "interval_close":time(hour=12, minute=30)})
-        times.append(time(hour=13, minute=30)) ; answers.append({"interval_open":time(hour=14, minute=0), "interval_close":time(hour=15, minute=30)})
-        times.append(time(hour=13, minute=45)) ; answers.append({"interval_open":time(hour=14, minute=0), "interval_close":time(hour=15, minute=30)})
+        times.append(time(9, 30))  ; answers.append({"interval_open":time(11, 0), "interval_close":time(12, 30)})
+        times.append(time(9, 45))  ; answers.append({"interval_open":time(11, 0), "interval_close":time(12, 30)})
+        times.append(time(13, 30)) ; answers.append({"interval_open":time(14, 0), "interval_close":time(15, 30)})
+        times.append(time(13, 45)) ; answers.append({"interval_open":time(14, 0), "interval_close":time(15, 30)})
         for i in range(len(times)):
             t = times[i]
             answer_t = answers[i]
@@ -550,9 +662,9 @@ class Test_Market_Intervals_USA(unittest.TestCase):
             interval_td = yfcd.intervalToTimedelta[interval]
 
             times = []
-            times.append(time(hour=15, minute=59))
+            times.append(time(15, 59))
             m = min(15, interval_td.seconds//60//2)
-            times.append(time(hour=15, minute=59-m))
+            times.append(time(15, 59-m))
             for d in [14,15,16,17,18]:
                 for t in times:
                     dt = datetime.combine(date(2022,2,d), t, self.market_tz)
@@ -564,7 +676,7 @@ class Test_Market_Intervals_USA(unittest.TestCase):
                     while not yfct.ExchangeOpenOnDay(self.exchange, next_day):
                         next_day += timedelta(days=1)
                     answer = {}
-                    answer["interval_open"] = datetime.combine(next_day, time(hour=9, minute=30), self.market_tz)
+                    answer["interval_open"] = datetime.combine(next_day, self.market_open_time, self.market_tz)
                     answer["interval_close"] = answer["interval_open"]+interval_td
                     if answer["interval_close"].time() > self.market_close_time:
                         answer["interval_close"] = datetime.combine(answer["interval_close"].date(), self.market_close_time, self.market_tz)
@@ -582,8 +694,8 @@ class Test_Market_Intervals_USA(unittest.TestCase):
         # If a day interval, is next working day regardless of today
         interval = yfcd.Interval.Days1
         times = []
-        times.append(time(hour=9, minute=30))
-        times.append(time(hour=15, minute=30))
+        times.append(self.market_open_time)
+        times.append(time(15, 30))
         for d in [14,15,16,17,18]:
             for t in times:
                 dt = datetime.combine(date(2022,2,d), t, self.market_tz)
@@ -626,8 +738,8 @@ class Test_Market_Intervals_USA(unittest.TestCase):
         intervals.append(yfcd.Interval.Days5)
         intervals.append(yfcd.Interval.Week)
         times = []
-        times.append(time(hour=9, minute=30))
-        times.append(time(hour=15, minute=30))
+        times.append(self.market_open_time)
+        times.append(time(15, 30))
         # Next week
         answer = {}
         answer["interval_open"] = date(2022,2,22) # Skip Monday because holiday
@@ -655,8 +767,8 @@ class Test_Market_Intervals_USA(unittest.TestCase):
         intervals.append(yfcd.Interval.Days5)
         intervals.append(yfcd.Interval.Week)
         times = []
-        times.append(time(hour=9, minute=30))
-        times.append(time(hour=15, minute=30))
+        times.append(self.market_open_time)
+        times.append(time(15, 30))
         # Next week
         answer = {}
         answer["interval_open"] = date(2022,2,21) # Start on Monday despite holiday
@@ -689,7 +801,7 @@ class Test_Market_Intervals_USA(unittest.TestCase):
         intervals.append(yfcd.Interval.Mins60)
         intervals.append(yfcd.Interval.Mins90)
         intervals.append(yfcd.Interval.Hours1)
-        t = time(hour=9, minute=0)
+        t = time(9,15)
         for i in range(len(intervals)):
             interval = intervals[i]
             interval_td = yfcd.intervalToTimedelta[interval]
@@ -699,9 +811,9 @@ class Test_Market_Intervals_USA(unittest.TestCase):
 
                 answer_day = date(2022,2,d)
                 answer = {}
-                answer["interval_open"]  = datetime.combine(answer_day, time(hour=9, minute=30), self.market_tz)
+                answer["interval_open"]  = datetime.combine(answer_day, self.market_open_time, self.market_tz)
                 if interval == yfcd.Interval.Days1:
-                    answer["interval_close"] = datetime.combine(answer_day, time(hour=16, minute=0), self.market_tz)
+                    answer["interval_close"] = datetime.combine(answer_day, self.market_close_time, self.market_tz)
                 else:
                     answer["interval_close"] = answer["interval_open"]+interval_td
                     if answer["interval_close"].time() > self.market_close_time:
@@ -727,7 +839,7 @@ class Test_Market_Intervals_USA(unittest.TestCase):
         intervals.append(yfcd.Interval.Mins60)
         intervals.append(yfcd.Interval.Mins90)
         intervals.append(yfcd.Interval.Hours1)
-        t = time(hour=16, minute=0)
+        t = self.market_close_time
         for i in range(len(intervals)):
             interval = intervals[i]
             interval_td = yfcd.intervalToTimedelta[interval]
@@ -741,7 +853,7 @@ class Test_Market_Intervals_USA(unittest.TestCase):
                 while not yfct.ExchangeOpenOnDay(self.exchange, next_day):
                     next_day += timedelta(days=1)
                 answer = {}
-                answer["interval_open"]  = datetime.combine(next_day, time(hour=9, minute=30), self.market_tz)
+                answer["interval_open"]  = datetime.combine(next_day, self.market_open_time, self.market_tz)
                 answer["interval_close"] = answer["interval_open"]+interval_td
                 try:
                     self.assertEqual(intervalRange, answer)
@@ -757,8 +869,8 @@ class Test_Market_Intervals_USA(unittest.TestCase):
         # If day interval, is next working day regardless of today
         interval = yfcd.Interval.Days1
         times = []
-        times.append(time(hour=9, minute=0))
-        times.append(time(hour=16, minute=0))
+        times.append(time(9, 0))
+        times.append(time(16, 0))
         for d in [14,15,16,17,18]:
             for t in times:
                 dt = datetime.combine(date(2022,2,d), t, self.market_tz)
@@ -788,9 +900,9 @@ class Test_Market_Intervals_USA(unittest.TestCase):
         intervals.append(yfcd.Interval.Days5)
         intervals.append(yfcd.Interval.Week)
         dates = []
-        dates.append(datetime.combine(date(2022,2,18), time(hour=16, minute=0), self.market_tz))
-        dates.append(datetime.combine(date(2022,2,19), time(hour=12, minute=0), self.market_tz))
-        dates.append(datetime.combine(date(2022,2,20), time(hour=12, minute=0), self.market_tz))
+        dates.append(datetime.combine(date(2022,2,18), time(16, 0), self.market_tz))
+        dates.append(datetime.combine(date(2022,2,19), time(12, 0), self.market_tz))
+        dates.append(datetime.combine(date(2022,2,20), time(12, 0), self.market_tz))
         for i in range(len(intervals)):
             interval = intervals[i]
             for dt in dates:
@@ -810,105 +922,13 @@ class Test_Market_Intervals_USA(unittest.TestCase):
                     raise
 
 
-    def test_GetExchangeScheduleIntervals_hourly(self):
-        interval = yfcd.Interval.Hours1
-
-        day = date(2022,2,14)
-
-        start_dt = datetime.combine(day, time(11,30), self.market_tz)
-        end_dt   = datetime.combine(day, time(15,30), self.market_tz)
-        answer = [(datetime.combine(day, time(11,30), self.market_tz), datetime.combine(day, time(12,30), self.market_tz)), 
-                  (datetime.combine(day, time(12,30), self.market_tz), datetime.combine(day, time(13,30), self.market_tz)),
-                  (datetime.combine(day, time(13,30), self.market_tz), datetime.combine(day, time(14,30), self.market_tz)),
-                  (datetime.combine(day, time(14,30), self.market_tz), datetime.combine(day, time(15,30), self.market_tz))]
-        response = yfct.GetExchangeScheduleIntervals(self.exchange, interval, start_dt, end_dt, weeklyUseYahooDef=False)
-        try:
-            self.assertEqual(response, answer)
-        except:
-            print("response:")
-            pprint(response)
-            print("answer:")
-            pprint(answer)
-            raise
-
-        start_dt = datetime.combine(day, time(11, 0), self.market_tz)
-        end_dt   = datetime.combine(day, time(15, 0), self.market_tz)
-        answer = [(datetime.combine(day, time(11,30), self.market_tz), datetime.combine(day, time(12,30), self.market_tz)), 
-                  (datetime.combine(day, time(12,30), self.market_tz), datetime.combine(day, time(13,30), self.market_tz)),
-                  (datetime.combine(day, time(13,30), self.market_tz), datetime.combine(day, time(14,30), self.market_tz))]
-        response = yfct.GetExchangeScheduleIntervals(self.exchange, interval, start_dt, end_dt, weeklyUseYahooDef=False)
-        try:
-            self.assertEqual(response, answer)
-        except:
-            print("response:")
-            pprint(response)
-            print("answer:")
-            pprint(answer)
-            raise
-
-        start_dt = datetime.combine(day, time(11,30), self.market_tz)
-        end_dt   = datetime.combine(day, time(18,30), self.market_tz)
-        answer = [(datetime.combine(day, time(11,30), self.market_tz), datetime.combine(day, time(12,30), self.market_tz)), 
-                  (datetime.combine(day, time(12,30), self.market_tz), datetime.combine(day, time(13,30), self.market_tz)),
-                  (datetime.combine(day, time(13,30), self.market_tz), datetime.combine(day, time(14,30), self.market_tz)),
-                  (datetime.combine(day, time(14,30), self.market_tz), datetime.combine(day, time(15,30), self.market_tz)),
-                  (datetime.combine(day, time(15,30), self.market_tz), datetime.combine(day, time(16, 0), self.market_tz))]
-        response = yfct.GetExchangeScheduleIntervals(self.exchange, interval, start_dt, end_dt, weeklyUseYahooDef=False)
-        try:
-            self.assertEqual(response, answer)
-        except:
-            print("response:")
-            pprint(response)
-            print("answer:")
-            pprint(answer)
-            raise
-
-    def test_GetExchangeScheduleIntervals_daily(self):
-        interval = yfcd.Interval.Days1
-
-        start_d = date(2022,2,14)
-        end_d = date(2022,2,19)
-        answer = [(date(2022,2,d),date(2022,2,d+1)) for d in [14,15,16,17,18]]
-        response = yfct.GetExchangeScheduleIntervals(self.exchange, interval, start_d, end_d, weeklyUseYahooDef=False)
-        try:
-            self.assertEqual(response, answer)
-        except:
-            print("response:")
-            pprint(response)
-            print("answer:")
-            pprint(answer)
-            raise
-
-    def test_GetExchangeScheduleIntervals_weekly(self):
-        intervals = [yfcd.Interval.Days5, yfcd.Interval.Week]
-        for interval in intervals:
-            start_d = date(2022,2,14)
-            end_d   = date(2022,2,27)
-            answer = [(date(2022,2,14), date(2022,2,19)),
-                      (date(2022,2,22), date(2022,2,26))]
-            response = yfct.GetExchangeScheduleIntervals(self.exchange, interval, start_d, end_d, weeklyUseYahooDef=False)
-            try:
-                self.assertEqual(response, answer)
-            except:
-                print("response:")
-                pprint(response)
-                print("answer:")
-                pprint(answer)
-                raise
-
-            start_d = date(2022,2,9)
-            end_d   = date(2022,3,1)
-            answer = [(date(2022,2,14), date(2022,2,19)),
-                      (date(2022,2,22), date(2022,2,26))]
-            response = yfct.GetExchangeScheduleIntervals(self.exchange, interval, start_d, end_d, weeklyUseYahooDef=False)
-            try:
-                self.assertEqual(response, answer)
-            except:
-                print("response:")
-                pprint(response)
-                print("answer:")
-                pprint(answer)
-                raise
-
 if __name__ == '__main__':
     unittest.main()
+
+    # # Run tests sequentially:
+    # import inspect
+    # test_src = inspect.getsource(Test_Market_Intervals_USA)
+    # unittest.TestLoader.sortTestMethodsUsing = lambda _, x, y: (
+    #     test_src.index(f"def {x}") - test_src.index(f"def {y}")
+    # )
+    # unittest.main(verbosity=2)
