@@ -42,7 +42,6 @@ import os
 
 ## TODO:
 ## Test for handling days without trades. Happens most on Toronto exchange
-## Test using 'tz' argument in history()
 
 class Test_Yfc_Interface(unittest.TestCase):
 
@@ -59,7 +58,8 @@ class Test_Yfc_Interface(unittest.TestCase):
         self.usa_tkr = "GME" # Stock split recently
         self.usa_market = "us_market"
         self.usa_exchange = "NMS"
-        self.usa_market_tz = ZoneInfo('US/Eastern')
+        self.usa_market_tz_name = 'US/Eastern'
+        self.usa_market_tz = ZoneInfo(self.usa_market_tz_name)
         self.usa_market_open_time  = time(hour=9, minute=30)
         self.usa_market_close_time = time(hour=16, minute=0)
         self.usa_dat = yfc.Ticker(self.usa_tkr, session=self.session)
@@ -78,6 +78,61 @@ class Test_Yfc_Interface(unittest.TestCase):
         self.session.close()
 
 
+    def verify_df(self, df, answer, rtol=None, different=False):
+        if df.shape[0] != answer.shape[0]:
+            raise Exception("Different #rows: df={}, answer={}".format(df.shape[0], answer.shape[0]))
+
+        dcs = ["Open","High","Low","Close","Volume","Dividends","Stock Splits"]
+        for dc in dcs:
+            if not (dc in df.columns and dc in answer.columns):
+                continue
+            if rtol is None:
+                f = (df[dc].values == answer[dc].values)
+            else:
+                f = np.isclose(df[dc].values, answer[dc].values, rtol=rtol)
+
+            if not different:
+                # Test that dfs are equal
+                try:
+                    self.assertTrue(f.all())
+                except:
+                    f_diff=~f ; f=None
+                    if sum(f_diff) < 20:
+                        print("{}/{} differences in column {}:".format(sum(f_diff), df.shape[0], dc))
+                        print("- answer:")
+                        print(answer[f_diff][[dc]])
+                        print("- result:")
+                        print(df[f_diff][[dc]])
+                    else:
+                        print("{}/{} diffs in column {}".format(sum(f_diff), df.shape[0], dc))
+
+                    last_diff_idx = np.where(f_diff)[0][-1]
+                    x = df[dc][last_diff_idx]
+                    y = answer[dc][last_diff_idx]
+                    last_diff = x - y
+                    print("- last_diff: {} - {} = {}".format(x, y, last_diff))
+                    print("- answer:")
+                    print(answer.iloc[last_diff_idx])
+                    print("- result:")
+                    print(df.iloc[last_diff_idx])
+                    raise
+
+            else:
+                # Test that dfs are different
+                try:
+                    self.assertFalse(f.all())
+                except:
+                    if sum(f) < 20:
+                        print("{}/{} matches in column {}:".format(sum(f), df.shape[0], dc))
+                        print("- answer:")
+                        print(answer[f][[dc]])
+                        print("- result:")
+                        print(df[f][[dc]])
+                    else:
+                        print("{}/{} matches in column {}".format(sum(f), df.shape[0], dc))
+                    raise
+
+
     def test_history_basics1_usa(self):
         # A fetch of prices, then another fetch of same prices, should return identical
 
@@ -164,7 +219,7 @@ class Test_Yfc_Interface(unittest.TestCase):
         self.assertEqual(df1.shape[0], 5)
         df2 = self.nze_dat.history(interval="1d", start=date(2022,2,8), end=date(2022,2,15))
         try:
-            self.assertTrue(df1.equals(df2))
+            self.verify_df(df1, df2)
         except:
             print("df1:")
             print(df1)
@@ -244,11 +299,14 @@ class Test_Yfc_Interface(unittest.TestCase):
         df1 = pd.concat([dfa,dfb])
 
         df2 = self.nze_dat.history(interval="1d", start=date(2022,2,8), end=date(2022,2,10))
-        self.assertTrue(df1.equals(df2))
+        self.verify_df(df1, df2)
 
 
     def test_history_basics_hour_usa(self):
         # Check fetching single hour
+
+        yfct.SetExchangeTzName(self.usa_exchange, self.usa_market_tz_name)
+
         start_d = date.today() - timedelta(days=1)
         while not yfct.ExchangeOpenOnDay(self.usa_exchange, start_d):
             start_d -= timedelta(days=1)
@@ -272,7 +330,7 @@ class Test_Yfc_Interface(unittest.TestCase):
         self.assertEqual(df1.shape[0], 1)
         self.assertEqual(df1.index[0], start_dt)
         df2 = self.nze_dat.history(interval="1h", start=start_dt, end=end_dt)
-        self.assertTrue(df1.equals(df2))
+        self.verify_df(df1, df2)
 
 
     def test_history_overlaps_usa(self):
@@ -297,11 +355,11 @@ class Test_Yfc_Interface(unittest.TestCase):
         df1 = self.nze_dat.history(interval="1d", start=date(2022,2,8), end=date(2022,2,11))
         df2 = self.nze_dat.history(interval="1d", start=date(2022,2,9), end=date(2022,2,12))
         ## Where these overlap should be identical
-        self.assertTrue(df1.iloc[1:3].reset_index(drop=True).equals(df2.iloc[0:2].reset_index(drop=True)))
+        self.verify_df(df1.iloc[1:3], df2.iloc[0:2])
 
         ## Then fetching full date range is identical to unique rows in df1+df2
         df3 = self.nze_dat.history(interval="1d", start=date(2022,2,8), end=date(2022,2,12))
-        self.assertTrue(df3.equals(pd.concat([df1.iloc[0:1], df2])))
+        self.verify_df(df3, pd.concat([df1.iloc[0:1], df2]))
 
 
     def test_matches_yf_daily_usa(self):
@@ -369,7 +427,7 @@ class Test_Yfc_Interface(unittest.TestCase):
 
     def test_history_final(self):
         # Test 'Final?' column
-        tkr_candidates = ["IMP.JO", "INTC", "MEL.NZ"]
+        tkr_candidates = ["BHG.JO", "INTC", "MEL.NZ"]
         interval = yfcd.Interval.Days1
 
         dt_now_utc = datetime.utcnow()
@@ -420,7 +478,7 @@ class Test_Yfc_Interface(unittest.TestCase):
 
     def test_periods(self):
         #
-        tkrs = ["MEL.NZ", "IMP.JO", "INTC"]
+        tkrs = ["MEL.NZ", "BHG.JO", "INTC"]
         tkrs.append("HLTH") # Listed recently
         tkrs.append("GME") # Stock split recently
         periods = [p for p in yfcd.Period]
@@ -432,7 +490,14 @@ class Test_Yfc_Interface(unittest.TestCase):
                 yfcm.SetCacheDirpath(self.tempCacheDir.name)
                 dat_yfc = yfc.Ticker(tkr, session=self.session)
 
-                df_yf = dat_yf.history(period=yfcd.periodToString[p], auto_adjust=False)
+                try:
+                    df_yf = dat_yf.history(period=yfcd.periodToString[p], auto_adjust=False)
+                except Exception as e:
+                    if "No data found for this date range" in str(e):
+                        # Skip
+                        continue
+                    else:
+                        raise
                 # Remove any rows when exchange was closed. Yahoo can be naughty and fill in rows when exchange closed.
                 sched = yfct.GetExchangeSchedule(dat_yfc.info["exchange"], df_yf.index[0].date(), df_yf.index[-1].date()+timedelta(days=1))
                 df_yf = df_yf[np.isin(df_yf.index.date, sched["market_open"].dt.date)]
@@ -459,7 +524,7 @@ class Test_Yfc_Interface(unittest.TestCase):
                     print("Different shapes")
                     raise
                 try:
-                    self.assertTrue(df_yf.index.equals(df_yfc.index))
+                    self.assertTrue(np.equal(df_yf.index, df_yfc.index).all())
                 except:
                     print("df_yf:")
                     print(df_yf)
@@ -506,7 +571,7 @@ class Test_Yfc_Interface(unittest.TestCase):
                     print("Different shapes")
                     raise
                 try:
-                    self.assertTrue(df_yf.index.equals(df_yfc.index))
+                    self.assertTrue(np.equal(df_yf.index, df_yfc.index).all())
                 except:
                         print("df_yf:")
                         print(df_yf)
@@ -538,7 +603,7 @@ class Test_Yfc_Interface(unittest.TestCase):
                         raise
 
     def test_periods_with_persistent_caching(self):
-        tkrs = ["MEL.NZ", "IMP.JO", "INTC"]
+        tkrs = ["MEL.NZ", "BHG.JO", "INTC"]
         tkrs.append("HLTH") # Listed recently
         tkrs.append("GME") # Stock split recently
         periods = [p for p in yfcd.Period]
@@ -547,7 +612,14 @@ class Test_Yfc_Interface(unittest.TestCase):
             dat_yf = yf.Ticker(tkr, session=self.session)
             dat_yfc = yfc.Ticker(tkr, session=self.session)
             for p in periods:
-                df_yf = dat_yf.history(period=yfcd.periodToString[p], auto_adjust=False)
+                try:
+                    df_yf = dat_yf.history(period=yfcd.periodToString[p], auto_adjust=False)
+                except Exception as e:
+                    if "No data found for this date range" in str(e):
+                        # Skip
+                        continue
+                    else:
+                        raise
                 # Remove any rows when exchange was closed. Yahoo can be naughty and fill in rows when exchange closed.
                 sched = yfct.GetExchangeSchedule(dat_yfc.info["exchange"], df_yf.index[0].date(), df_yf.index[-1].date()+timedelta(days=1))
                 df_yf = df_yf[np.isin(df_yf.index.date, sched["market_open"].dt.date)]
@@ -573,14 +645,14 @@ class Test_Yfc_Interface(unittest.TestCase):
                     print("Different shapes")
                     raise
                 try:
-                    self.assertTrue(df_yf.index.equals(df_yfc.index))
+                    self.assertTrue(np.equal(df_yf.index, df_yfc.index).all())
                 except:
-                        print("df_yf:")
-                        print(df_yf)
-                        print("df_yfc:")
-                        print(df_yfc)
-                        print("Index different")
-                        raise
+                    print("df_yf:")
+                    print(df_yf)
+                    print("df_yfc:")
+                    print(df_yfc)
+                    print("Index different")
+                    raise
                 f = df_yfc["Final?"].values
                 for c in yfcd.yf_data_cols:
                     if not c in df_yf.columns:
@@ -619,7 +691,7 @@ class Test_Yfc_Interface(unittest.TestCase):
                     print("Different shapes")
                     raise
                 try:
-                    self.assertTrue(df_yf.index.equals(df_yfc.index))
+                    self.assertTrue(np.equal(df_yf.index, df_yfc.index).all())
                 except:
                         print("df_yf:")
                         print(df_yf)
@@ -653,7 +725,7 @@ class Test_Yfc_Interface(unittest.TestCase):
 
     def test_history_live(self):
         # Fetch during live trading session
-        tkr_candidates = ["IMP.JO", "INTC", "MEL.NZ"]
+        tkr_candidates = ["BHG.JO", "INTC", "MEL.NZ"]
         interval = yfcd.Interval.Days1
 
         dt_now = datetime.utcnow().replace(tzinfo=ZoneInfo("UTC"))
@@ -669,7 +741,6 @@ class Test_Yfc_Interface(unittest.TestCase):
             if yfct.IsTimestampInActiveSession(exchange, dt_now):
                 d = start_dt.date()
                 expected_interval_dates = []
-                # tz = pytz.timezone(dat.info["exchangeTimezoneName"])
                 tz = ZoneInfo(dat.info["exchangeTimezoneName"])
                 while d < end_dt.date():
                     if yfct.ExchangeOpenOnDay(exchange, d):
@@ -677,8 +748,10 @@ class Test_Yfc_Interface(unittest.TestCase):
                         expected_interval_dates.append(dt)
                     d += timedelta(days=1)
 
-                ## Check that table covers expected date range
                 df1 = dat.history(interval="1d", start=start_d, end=end_d)
+                df_yf = yf.Ticker(tkr, session=self.session).history(interval="1d", start=start_d, end=end_d)
+
+                ## Check that table covers expected date range
                 try:
                     self.assertTrue(np.array_equal(expected_interval_dates, df1.index))
                 except:
@@ -693,42 +766,44 @@ class Test_Yfc_Interface(unittest.TestCase):
                     print(expected_interval_dates == df1.index)
                     raise
 
+                ## Check it matches YF:
+                self.verify_df(df1, df_yf)
+
                 # Refetch before data aged, should return identical table
                 sleep(1)
                 df2 = dat.history(interval="1d", start=start_d, end=end_d, max_age=timedelta(minutes=1))
-                try:
-                    self.assertTrue(df1.equals(df2))
-                except:
-                    for c in df1.columns:
-                        if not df1[c].equals(df2[c]):
-                            print("Column {} different".format(c))
-                            print(df1[c])
-                            print(df2[c])
-                            print(df1[c]==df2[c])
-                            break
-                    print("df1:")
-                    pprint(df1)
-                    print("df1 cols: {}".format(["'{}'".format(x) for x in df1.columns]))
-                    print("df2:")
-                    print(df2)
-                    print("df2 cols: {}".format(["'{}'".format(x) for x in df2.columns]))
-                    raise
+                self.verify_df(df1, df2)
 
                 # Refetch after data aged, last row should be different
-                sleep(10)
+                sleep(20)
                 n = df1.shape[0]
                 df3 = dat.history(interval="1d", start=start_d, end=end_d, max_age=timedelta(seconds=1))
+                try:
+                    self.assertEqual(len(expected_interval_dates), df3.shape[0])
+                except:
+                    print("Different shapes")
+                    print("df1:")
+                    print(df1)
+                    print("df3:")
+                    print(df3)
+                    raise
                 try:
                     self.assertTrue(np.array_equal(expected_interval_dates, df3.index))
                 except:
                     print("expected_interval_dates:")
                     pprint(expected_interval_dates)
                     print("df3:")
-                    print(df3)
+                    print(df3.index)
+                    edt0 = expected_interval_dates[0]
+                    adt0 = df3.index[0]
+                    print("expected dt0 = {} (tz={})".format(edt0, edt0.tzinfo))
+                    print("actual dt0 = {} (tz={})".format(adt0, adt0.tzinfo))
+                    print(expected_interval_dates == df3.index)
                     raise
                 try:
-                    self.assertTrue(df1.iloc[0:n-1].equals(df3.iloc[0:n-1]))
-                    self.assertFalse(df1.iloc[n-1:n].equals(df3.iloc[n-1:n]))
+                    self.verify_df(df1.iloc[0:n-1], df3.iloc[0:n-1])
+                    # Restrict differences to volume column, sometimes price hasn't moved
+                    self.verify_df(df1[["Volume"]].iloc[n-1:n], df3[["Volume"]].iloc[n-1:n], different=True)
                 except:
                     print("df1:")
                     print(df1)
@@ -736,30 +811,9 @@ class Test_Yfc_Interface(unittest.TestCase):
                     print(df3)
                     raise
 
-                ## Check it matches YF:
-                dat_yf = yf.Ticker(tkr, session=self.session)
-                df_yf = dat_yf.history(interval="1d", start=start_d, end=end_d)
-                for c in yfcd.yf_data_cols:
-                    if not c in df_yf.columns:
-                        continue
-                    elif c == "Adj Close" and not c in df_yfc.columns:
-                        continue
-                    try:
-                        self.assertTrue(np.array_equal(df_yf[c].values, df1[c].values))
-                    except:
-                        print("")
-                        print("df_yf:")
-                        print(df_yf[[c]])
-                        print("")
-                        print("df_yfc:")
-                        print(df1[[c]])
-                        print("")
-                        print("Difference in column {}".format(c))
-                        raise
-
     def test_history_live_1h_evening(self):
         # Fetch during evening after active session
-        tkr_candidates = ["IMP.JO", "INTC", "MEL.NZ"]
+        tkr_candidates = ["BHG.JO", "INTC", "MEL.NZ"]
         interval = yfcd.Interval.Hours1
         #
         dt_now = datetime.utcnow().replace(tzinfo=ZoneInfo("UTC"))
@@ -776,6 +830,9 @@ class Test_Yfc_Interface(unittest.TestCase):
                     start_dt = sched["market_open"][0].to_pydatetime()
                     end_dt = sched["market_close"][0].to_pydatetime()
 
+                    # Ensure Yahoo returns correct volume for first interval:
+                    start_dt -= timedelta(minutes=20)
+
                     dt = sched["market_open"][0]
                     expected_interval_starts = []
                     while dt < sched["market_close"][0]:
@@ -796,27 +853,30 @@ class Test_Yfc_Interface(unittest.TestCase):
                     ## Finally, check it matches YF:
                     dat_yf = yf.Ticker(tkr, session=self.session)
                     df_yf = dat_yf.history(interval="1h", start=start_dt, end=end_dt)
-                    for c in yfcd.yf_data_cols:
-                        if not c in df_yf.columns:
-                            continue
-                        elif c == "Adj Close" and not c in df_yfc.columns:
-                            continue
-                        try:
-                            self.assertTrue(np.array_equal(df_yf[c].values, df1[c].values))
-                        except:
-                            print("")
-                            print("df_yf:")
-                            print(df_yf[[c]])
-                            print("")
-                            print("df_yfc:")
-                            print(df1[[c]])
-                            print("")
-                            print("{}: Difference in column {}".format(tkr, c))
-                            raise
+                    # Note: Yahoo doesn't dividend-adjust hourly
+                    df1 = dat.history(start=start_dt, end=end_dt, interval="1h", adjust_divs=False)
+                    df_yf = dat_yf.history(start=start_dt, interval="1h")
+                    # Discard 0-volume data at market close
+                    td_1d = timedelta(days=1)
+                    sched = yfct.GetExchangeSchedule(dat.info["exchange"], df_yf.index.date.min(), df_yf.index.date.max()+td_1d)
+                    sched["_date"] = sched.index.date
+                    df_yf["_date"] = df_yf.index.date
+                    answer2 = df_yf.merge(sched, on="_date", how="left", validate="many_to_one")
+                    answer2.index = df_yf.index ; df_yf = answer2
+                    f_drop = (df_yf["Volume"]==0).values & ((df_yf.index<df_yf["market_open"]) | (df_yf.index>=df_yf["market_close"]))
+                    df_yf = df_yf[~f_drop].drop("_date",axis=1)
+                    # YF hourly volume is not split-adjusted, so adjust:
+                    ss = df_yf["Stock Splits"].copy()
+                    ss[ss==0.0] = 1.0
+                    ss_rcp = 1.0/ss
+                    csf = ss_rcp.sort_index(ascending=False).cumprod().sort_index(ascending=True).shift(-1, fill_value=1.0)
+                    df_yf["Volume"] /= csf
+                    df_yf = df_yf[df_yf.index<end_dt]
+                    self.verify_df(df1, df_yf, 1e-10)
 
     def test_history_live_1d_evening(self):
         # Fetch during evening after active session
-        tkr_candidates = ["IMP.JO", "INTC", "MEL.NZ"]
+        tkr_candidates = ["BHG.JO", "INTC", "MEL.NZ"]
         interval = yfcd.Interval.Days1
         #
         dt_now = datetime.utcnow().replace(tzinfo=ZoneInfo("UTC"))
@@ -854,27 +914,11 @@ class Test_Yfc_Interface(unittest.TestCase):
                     ## Finally, check it matches YF:
                     dat_yf = yf.Ticker(tkr, session=self.session)
                     df_yf = dat_yf.history(interval="1d", start=start_dt.date(), end=end_dt.date())
-                    for c in yfcd.yf_data_cols:
-                        if not c in df_yf.columns:
-                            continue
-                        elif c == "Adj Close" and not c in df_yfc.columns:
-                            continue
-                        try:
-                            self.assertTrue(np.array_equal(df_yf[c].values, df1[c].values))
-                        except:
-                            print("")
-                            print("df_yf:")
-                            print(df_yf)
-                            print("")
-                            print("df_yfc:")
-                            print(df1)
-                            print("")
-                            print("Difference in column {}".format(c))
-                            raise
+                    self.verify_df(df1, df_yf, 1e-10)
 
     def test_history_live_1w_evening(self):
         # Fetch during evening after active session
-        tkr_candidates = ["IMP.JO", "INTC"]
+        tkr_candidates = ["BHG.JO", "INTC"]
         interval = yfcd.Interval.Week
         #
         dt_now = datetime.utcnow().replace(tzinfo=ZoneInfo("UTC"))
@@ -918,23 +962,7 @@ class Test_Yfc_Interface(unittest.TestCase):
                     ## Finally, check it matches YF:
                     dat_yf = yf.Ticker(tkr, session=self.session)
                     df_yf = dat_yf.history(interval="1wk", start=start_dt.date(), end=end_dt.date())
-                    for c in yfcd.yf_data_cols:
-                        if not c in df_yf.columns:
-                            continue
-                        elif c == "Adj Close" and not c in df_yfc.columns:
-                            continue
-                        try:
-                            self.assertTrue(np.array_equal(df_yf[c].values, df1[c].values))
-                        except:
-                            print("Difference in column {}".format(c))
-                            print("")
-                            print("df_yf:")
-                            print(df_yf)
-                            print("")
-                            print("df_yfc:")
-                            print(df1)
-                            print("")
-                            raise
+                    self.verify_df(df1, df_yf)
 
     def test_history_jse_is_weird(self):
         # JSE market closes at 5pm local time, but if request 1h data 
@@ -942,21 +970,25 @@ class Test_Yfc_Interface(unittest.TestCase):
         # This mysterious row disappears tomorrow.
         interval = "1h"
         tkr = "IMP.JO"
-
-        dt_now = datetime.utcnow().replace(tzinfo=ZoneInfo("UTC"))
-        d_now = dt_now.date()
         dat = yfc.Ticker(tkr, session=None)
         exchange = dat.info["exchange"]
-        yfct.SetExchangeTzName(exchange, dat.info["exchangeTimezoneName"])
+        tz_name = dat.info["exchangeTimezoneName"]
+        tz = ZoneInfo(tz_name)
+        yfct.SetExchangeTzName(exchange, tz_name)
+
+        dt_now = datetime.utcnow().replace(tzinfo=ZoneInfo("UTC"))
+        # d_now = dt_now.date()
+        d_now = dt_now.astimezone(tz).date()
+        td_1d = timedelta(days=1)
         if yfct.ExchangeOpenOnDay(exchange, d_now):
-            sched = yfct.GetExchangeSchedule(exchange, d_now, d_now+timedelta(days=1))
-            if (not sched is None) and dt_now > sched["market_close"][0]:
-                start_dt = dt_now - timedelta(days=7)
-                end_dt = dt_now
-                tz = ZoneInfo(dat.info["exchangeTimezoneName"])
+            sched = yfct.GetExchangeSchedule(exchange, d_now-5*td_1d, d_now+td_1d)
+            if not sched is None:
+                for i in range(sched.shape[0]-1, -1, -1):
+                    if dt_now >= sched["market_close"].iloc[i]:
+                        break
+                d = sched["market_close"].iloc[i].date()
+                df = dat.history(start=d, end=d+td_1d, interval="1h")
 
-                ## Test is simple. Fetch should complete, and no 5pm in table
-                df = dat.history(start=d_now, interval="1h")
                 try:
                     self.assertEqual(df.shape[0], 8)
                     self.assertEqual(df.index[-1].time(), time(16))
